/***************************************************************************
 *   Copyright (C) 2005-2007 by Christian Weilbach                         *
 *   christian_weilbach@web.de                                             *
 *   Copyright (C) 2007 by Antonio Aloisio                                 *
 *   gnuton@gnuton.org                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qsize.h>
#include <qpixmap.h>
#include <qcursor.h>

#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kaboutdialog.h>
#include <kbugreport.h>
#include <kglobal.h>
#include <klocale.h>
#include <kconfigdialog.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kstandarddirs.h>
#include <kurl.h>
#include <kio/job.h>
#include <kstartupinfo.h>
#include <kpassivepopup.h>
#include <kiconloader.h>

#include "kblogger.h"

#include "kbloggermenu.h"
#include "kbloggerconfigdialog.h"
#include "kbloggerconfig.h"
#include "kbloggereditor.h"
#include "kbloggerupload.h"
#include "API_Blogger.h"
#include "API_MetaWeblog.h"

const int BLOGGER_API=0;
const int METAWEBLOG_API=1;

class KBloggerMenu;

KBlogger::KBlogger ( const QString& configFile, Type type, int actions, QWidget *parent, const char *name )
		: KPanelApplet ( configFile, type, actions, parent, name ), m_editor(0), m_menu(0), m_tempDir(0)

{
	m_iconLoader=KGlobal::iconLoader();

	// create Temp directory
	m_tempDir= new KTempDir(locateLocal("tmp", "kblogger",true));
	m_tempDir->setAutoDelete (true);

	// create backend
	updateBackend();

	// create menu
	m_menu=new KBloggerMenu ( m_backend, this, "menu" );

	// create the applet-widget
	kdDebug() << "KBlogger::KBlogger - Creating Applet-widget" << endl;

	// create the panel button.
// 	panelIcon= new QPixmap( m_iconLoader->loadIcon("kblogger",KIcon::Panel ) );
	m_appletlay = new QVBoxLayout ( this );
// 	m_label= new QLabel( this );
// 	m_label->setPixmap(*panelIcon);
	m_button = new QPushButton ( i18n ( "Blog" ),this );
	m_appletlay->addWidget ( m_button );
	connect ( m_button, SIGNAL ( clicked() ), this, SLOT ( slotClicked() ) );
	if ( !parent )
		setBackgroundMode ( X11ParentRelative );

	setAcceptDrops ( true );

	// initialize about data
	m_aboutData = new KAboutData ( "kblogger", I18N_NOOP ( "KBlogger - Your voice on the Net" ),
	                               "0.7beta2", I18N_NOOP ( "A kicker applet for blogging" ), KAboutData::License_GPL,
	                               0, 0, "http://kblogger.pwsp.net", "christian_weilbach@web.de" );
	m_aboutData->addAuthor ( "Christian Weilbach", I18N_NOOP ( "Main developer" ),
	                         I18N_NOOP ( "christian_weilbach@web.de" ), I18N_NOOP ( "http://whiletaker.homeip.net" ) );
	m_aboutData->addAuthor ( "Antonio 'GnuTon' Aloisio", I18N_NOOP ( "Developer" ),
	                         I18N_NOOP ( "gnuton@gnuton.org" ), I18N_NOOP ( "http://www.gnuton.org" ) );
	m_aboutData->addAuthor ( "Harald Sitter", I18N_NOOP ( "Documentation and Support" ) );
	m_aboutData->addAuthor ( "Kenneth Wimer", I18N_NOOP ( "Icon designer" ) );

	// initialize configuration
	kdDebug() << "KBlogger::KBlogger - Creating KConfigDialog"  <<  endl;
	m_configDialog = new KConfigDialog ( this, "settings", KBloggerConfig::self(), KDialogBase::Plain );
	m_configwd = new KBloggerConfigDialog();
	m_configDialog->addPage ( m_configwd, i18n ( "KBlogger settings" ), "KBlogger settings" );
	m_configDialog->setInitialSize ( QSize ( 520, 300 ), false ); // FIXME

	connect ( m_backend, SIGNAL ( error ( const QString& ) ),
	          this, SLOT ( handleBackendError ( const QString& ) ) );
	connect ( m_configDialog, SIGNAL ( settingsChanged() ),
	          this, SLOT ( updateBackend() ) );
	connect ( m_backend, SIGNAL ( mediaInfoRetrieved ( const QString& ) ),
	          this, SLOT ( handleSuccess ( const QString& ) ) );

	KStartupInfo::appStarted();
}

void KBlogger::slotClicked()
{
	editorSlot();
}

KBlogger::~KBlogger()
{
	kdDebug() << "KBlogger::~KBlogger()" << endl;
	delete m_backend;
	delete m_aboutData;
	delete m_configwd;
	delete m_configDialog;
	delete m_button;
	delete m_appletlay;
// 	delete m_label;
// 	delete panelIcon;
}

void KBlogger::handleSuccess ( const QString& string )
{
	KMessageBox::information ( 0, i18n ( "Element %1 posted." ).arg ( string ) );
}

void KBlogger::menuSlot()
{
	kdDebug() << "KBlogger::menuSlot()" << endl;
	if ( KBloggerConfig::type() != BLOGGER_API ) m_menu->setFullMenu ( true );
	m_menu->setPosition ( QCursor::pos() );
	m_menu->populateMenu();
}

void KBlogger::updateBackend()
{
	kdDebug() << "KBlogger::updateBackend()\n - Creating KURL..." << endl;
	KURL mServerUrl ( KBloggerConfig::url() );

	// create the backend
	kdDebug() << "  - API type =" << KBloggerConfig::type() << endl;
	switch ( KBloggerConfig::type() )
	{
		case BLOGGER_API:
			kdDebug() << "  - Creating APIBlogger( '"<< KBloggerConfig::url() << "' )" << endl;
			m_backend = new KBlog::APIBlogger ( mServerUrl );
			m_backend->setAppID ( "0123456789ABCDEF" ); // id not needed anymore
			break;

		case METAWEBLOG_API:
			m_backend = new KBlog::APIMetaWeblog ( mServerUrl );
			break;

		default:
		{
			QPixmap icon( m_iconLoader->loadIcon("kblogger/warning",KIcon::Panel ));
			KPassivePopup::message ( "KBlogger Warning","API Type not found. You have to configure first.",icon,this,"Passive Error Message" );
			m_backend=0;
			return;
			break;
		}
	}
	m_backend->setUsername ( KBloggerConfig::user() );
	m_backend->setPassword ( KBloggerConfig::password() );
	QPixmap icon( m_iconLoader->loadIcon("kblogger",KIcon::Panel ));
}


void KBlogger::editorSlot()
{
	kdDebug() << "KBlogger::editorSlot()" << endl;
	if (!m_editor){
	  m_editor = new KBloggerEditor ( this->m_backend,tmpDir(), -1,this );
	  connect (m_editor,SIGNAL(editorClosed()),SLOT(editorClosedSlot()));
	}/*else{
	  QPixmap icon( m_iconLoader->loadIcon("kblogger/warning.png",KIcon::Panel ));
	  KPassivePopup::message ( "KBlogger warning:","Another editor dialog is opened.",icon,this,"Passive Error Message" );
	}*/
}

void KBlogger::editorSlot ( int postid )
{
	kdDebug() << "KBlogger::editorSlot()" << endl;
// 	delete m_editor;
	if (!m_editor){
	  m_editor = new KBloggerEditor ( this->m_backend,tmpDir(), postid,this );
	  connect (m_editor,SIGNAL(editorClosed()),SLOT(editorClosedSlot()));
	  connect (this,SIGNAL(),m_editor,SLOT());
	}else{
	  QPixmap icon( m_iconLoader->loadIcon("kblogger/warning.png",KIcon::Panel ));
	  KPassivePopup::message ( "KBlogger warning:","Another editor dialog is opened.",icon,this,"Passive Error Message" );
	}
}

void KBlogger::uploadSlot ( QString filename )
{
	kdDebug() << "KBlogger::uploadSlot()" << endl;
	m_upload =  new KBloggerUpload ( ( KBlog::APIMetaWeblog* )m_backend, filename, KBloggerUpload::Generic, tmpDir(), this, "KBloggerUpload GUI" );
}

void KBlogger::uploadImageSlot ( QString filename )
{
	kdDebug() << "KBlogger::uploadSlot()" << endl;
	m_upload =  new KBloggerUpload (m_backend, filename, KBloggerUpload::Images, tmpDir(), this, "KBloggerUpload GUI" );
}

void KBlogger::uploadSlot()
{
	this->uploadSlot ( QString::null );
}


void KBlogger::handleBackendError ( const QString& error )
{
	if ( !error ) return;
	if ( !m_editor )
	{
		QPixmap icon( m_iconLoader->loadIcon("kblogger/warning.png",KIcon::Panel ));
		kdDebug() << "KBlogger::handleBackendError - error " << error << endl;
		KPassivePopup::message ( "KBlogger error:",error,icon,this,"Passive Error Message" );
	}
}
void KBlogger::editorClosedSlot()
{
  disconnect (m_editor,SIGNAL(editorClosed()),this,SLOT(editorClosedSlot()));
  m_editor=0;
}

void KBlogger::about()
{
	kdDebug() << "KBlogger::about()" << endl;

	KAboutApplication aboutdialog ( m_aboutData );
	aboutdialog.exec();
}

void KBlogger::reportBug()
{
	KBugReport bugReport ( this, true, m_aboutData );
	bugReport.exec();
}

void KBlogger::help()
{
	kapp->invokeHelp ( QString::null, QString::fromLatin1 ( "kblogger" ) );
}


void KBlogger::preferences()
{
	m_configDialog->show();
}

int KBlogger::widthForHeight( int height ) const
{
// 	return panelIcon->width();
	return 65;
}

int KBlogger::heightForWidth( int width ) const
{
// 	return panelIcon->height();
	return 40;
}

void KBlogger::mousePressEvent ( QMouseEvent *e )
{
	if ( e->button() == QMouseEvent::RightButton )
		menuSlot();
	if ( e->button() == QMouseEvent::LeftButton )
		slotClicked();
}


extern "C"
{
	KPanelApplet* init ( QWidget *parent, const QString& configFile )
	{
		KGlobal::locale()->insertCatalogue ( "kblogger" );
		return new KBlogger ( configFile, KPanelApplet::Normal,
		                      KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
		                      parent, "kblogger" );
	}
}
