/***************************************************************************
 *   Copyright (C) 2005-2007 by Christian Weilbach                         *
 *   christian_weilbach@web.de                                             *
 *   Copyright (C) 2007 by Antonio Aloisio                                 *
 *   gnuton@gnuton.org                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KBLOGGEREDITOR_H
#define KBLOGGEREDITOR_H

#include <qdir.h>

#include <ktextedit.h>

#include "kbloggereditorblank.h"
#include "API_Blog.h"

class KToolBar;
class KActionCollection;
class KStatusBar;
class KBlogger;
class KHTMLPart;
class KURL;
class KBloggerLinkDialog;


class KBloggerEditor : public KBloggerEditorBlank
{
	Q_OBJECT

public:
	KBloggerEditor( KBlog::APIBlog* backend, QDir* tempDir, int postid = -1, QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~KBloggerEditor();
	enum tags{ generic=0, img, imglinked, link, notags, };

public slots:
	virtual void sendSlot();
	virtual void getCategoriesSlot( const QString& name, const QString& description );

protected slots:
	virtual void setCategDesc(int);
	void writeOnPreview(const QString& text);

private:
	//Vars
	KBlog::APIBlog* m_backend;
	int m_postid;
	QString m_editCategory;
	KBloggerLinkDialog *m_linkDialog;
	KToolBar *m_toolBar;
	KActionCollection *m_actionCollection; //FIXME? Use a application Action collection?
	KStatusBar *m_statusBar;
	KBlogger *m_parent;
	QStringList categDescription;
	int nCategFetched;
	KHTMLPart *PreviewBrowser;
	const KURL *cssUrl;
	QDir *m_tempDir;

	//Functions
	void insertTag(const QString& textA, const QString& textB=0, const QString& textC=0,tags tag=notags); //Add a richText in the QEditBox at current position.
	QString richtextToHtml(const QString& text); //convert a <span... in <b, <i, <u....
	QString htmlToRichtext(const QString& text);
	//QString sanitizeText(const QString& text); //Remove spellchecking tags, richtext tags, convert "richtext to html"
	virtual bool close(bool destruct); //Override QWidget::Close(bool)

private slots:
	void loadPostSlot(KBlog::BlogPosting &posting);
	void disconnectCategoriesSlot();

	//Visual Editor private slots.
	void toggleItalic();
	void toggleBold();
	void toggleUnderline();
	void toggleStrikeOut();
	void linkDialogSlot();
	void htmlDialogSlot();
	void toggleCode();
	void addImage();
	void addImageUploaded( const QString& url );
	void addHtmlLink(const QString &, const QString& );
	void spellchecking(){ VisualTextEditor->checkSpelling(); }
	void closeSignal( const int );
	void handleSuccess( const int );
	void handleBackendError( const QString& );

	virtual void syncEditors(QWidget* m_tab);

signals:
	void editorClosed();

};

#endif

